package net.knarcraft.jarupdater;

import net.knarcraft.jarupdater.utility.DownloaderUtil;
import net.knarcraft.jarupdater.validation.DownloadURLValidator;
import net.knarcraft.jarupdater.validation.Validator;

import java.io.File;
import java.io.IOException;
import java.nio.file.Paths;
import java.util.concurrent.TimeUnit;

public class Main {

    private static final String usageString = "Usage: downloadURL [runAfterDownload (y/n)] [targetFile] [delay]\nThe " +
            "downloaded file will be run if runAfterDownload is given as y/yes. The default target will be the file " +
            "name defined in the download URL. The default delay is 5 seconds, and is the time to wait for the " +
            "calling .jar file to be writable.";
    private static final String invalidTargetFilename = "Target filename is invalid. Please set a targetFile argument" +
            " ending with a three-letter extension";
    private static final String invalidDownloadURL = "Invalid download URL. Only URLs pointing to knarcraft.net can " +
            "be used.";

    private static String downloadURL;
    private static String targetFile;
    private static boolean runAfterUpdate = false;
    private static int delay = 5;

    public static void main(String[] args) throws InterruptedException, IOException {
        parseAndValidateArguments(args);
        TimeUnit.SECONDS.sleep(delay);

        //Try to download the updated file
        boolean success = DownloaderUtil.downloadFile(downloadURL,
                Paths.get(DownloaderUtil.getApplicationWorkDirectory() + File.separator + targetFile));

        //Run the downloaded file to start the updated software
        if (success && runAfterUpdate) {
            DownloaderUtil.runExecutable(targetFile);
        }
    }

    /**
     * Parses and validates command line arguments
     *
     * @param args <p>The arguments given by the user</p>
     */
    private static void parseAndValidateArguments(String[] args) {
        if (args.length >= 2) {
            runAfterUpdate = args[1].toLowerCase().startsWith("y");
        }

        if (args.length >= 4) {
            delay = Integer.parseInt(args[3]);
        }

        if (args.length >= 1) {
            downloadURL = args[0];
        } else {
            System.out.println(usageString);
            System.exit(1);
            return;
        }

        if (args.length >= 3) {
            targetFile = args[2];
        } else {
            targetFile = DownloaderUtil.getFileNameFromURL(downloadURL);
            if (!DownloaderUtil.stringEndsWithExtension(targetFile)) {
                throw new IllegalArgumentException(invalidTargetFilename);
            }
        }

        Validator<String> urlValidator = new DownloadURLValidator();
        if (!urlValidator.validate(downloadURL)) {
            throw new IllegalArgumentException(invalidDownloadURL);
        }
    }

}
